## $Id: browser.py 10692 2013-11-04 10:57:34Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import grok
from zope.i18n import translate
from zope.component import getUtility
from hurry.workflow.interfaces import IWorkflowInfo
from waeup.kofa.interfaces import ADMITTED, IKofaUtils
from waeup.kofa.browser.layout import UtilityView
from waeup.kofa.students.interfaces import IStudentsUtils
from waeup.kofa.widgets.datewidget import FriendlyDatetimeDisplayWidget
from waeup.kofa.students.browser import (
    StartClearancePage, BedTicketAddPage, ExportPDFAdmissionSlipPage,
    StudentBasePDFFormPage)
from waeup.kwarapoly.students.interfaces import (
    ICustomStudent, ICustomStudentBase, ICustomStudentStudyLevel)
from waeup.kwarapoly.interfaces import MessageFactory as _
from waeup.kofa.students.workflow import (
    ADMITTED, PAID, REQUESTED, RETURNING, CLEARED, REGISTERED,
    VALIDATED, GRADUATED, TRANSCRIPT, CREATED, CLEARANCE)
from kofacustom.nigeria.students.browser import (
    NigeriaOnlinePaymentDisplayFormPage,
    NigeriaOnlinePaymentAddFormPage,
    NigeriaExportPDFPaymentSlipPage,
    NigeriaStudentClearanceEditFormPage,
    NigeriaExportPDFCourseRegistrationSlipPage,
    )

from waeup.kwarapoly.students.interfaces import ICustomStudentOnlinePayment

class CustomOnlinePaymentDisplayFormPage(NigeriaOnlinePaymentDisplayFormPage):
    """ Page to view an online payment ticket
    """
    grok.context(ICustomStudentOnlinePayment)
    form_fields = grok.AutoFields(ICustomStudentOnlinePayment).omit(
        'provider_amt', 'gateway_amt', 'thirdparty_amt', 'p_item')
    form_fields[
        'creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields[
        'payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')

class CustomOnlinePaymentAddFormPage(NigeriaOnlinePaymentAddFormPage):
    """ Page to add an online payment ticket
    """
    form_fields = grok.AutoFields(ICustomStudentOnlinePayment).select(
        'p_category')

class CustomExportPDFPaymentSlipPage(NigeriaExportPDFPaymentSlipPage):
    """Deliver a PDF slip of the context.
    """
    grok.context(ICustomStudentOnlinePayment)
    form_fields = grok.AutoFields(ICustomStudentOnlinePayment).omit(
        'provider_amt', 'gateway_amt', 'thirdparty_amt', 'p_item')
    form_fields['creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields['payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')

class CustomStartClearancePage(StartClearancePage):

    with_ac = False

class CustomStudentClearanceEditFormPage(NigeriaStudentClearanceEditFormPage):
    """ View to edit student clearance data by student
    """

    def dataNotComplete(self):
        return False

class BedTicketAddPage(BedTicketAddPage):
    """ Page to add an online payment ticket
    """
    buttonname = _('Create bed ticket')
    notice = ''
    with_ac = False

class CustomExportPDFAdmissionSlipPage(ExportPDFAdmissionSlipPage):
    """Deliver a PDF Admission slip.
    """
    grok.context(ICustomStudent)

    omit_fields = ('date_of_birth',)

    form_fields = grok.AutoFields(ICustomStudent).select('student_id', 'reg_number')

    def render(self):
        if self.context.state in (CREATED, ADMITTED,
                                  CLEARANCE, REQUESTED, CLEARED):
            self.flash('Not allowed.')
            self.redirect(self.url(self.context))
            return
        students_utils = getUtility(IStudentsUtils)
        return students_utils.renderPDFAdmissionLetter(self,
            self.context.student, omit_fields=self.omit_fields)

class ExportPDFAdmissionNotificationPage(UtilityView, grok.View):
    """Deliver a PDF Admission nostification slip.
    """
    grok.context(ICustomStudent)
    grok.name('admission_notification.pdf')
    grok.require('waeup.viewStudent')
    prefix = 'form'
    label = 'Notification of Provisional Admission'

    omit_fields = ('date_of_birth',)

    form_fields = grok.AutoFields(ICustomStudent).select(
        'student_id', 'reg_number', 'sex', 'lga')

    def render(self):
        if self.context.state not in (ADMITTED, CLEARANCE, REQUESTED, CLEARED):
            self.flash('Not allowed.')
            self.redirect(self.url(self.context))
            return
        students_utils = getUtility(IStudentsUtils)
        pre_text = ''
        post_text = '<strong>Instructions for Freshers</strong>'
        return students_utils.renderPDFAdmissionLetter(self,
            self.context.student, omit_fields=self.omit_fields,
            pre_text=pre_text, post_text=post_text)

# copied from waeup.aaue
class CustomExportPDFCourseRegistrationSlipPage(
    NigeriaExportPDFCourseRegistrationSlipPage):
    """Deliver a PDF slip of the context.
    """
    grok.context(ICustomStudentStudyLevel)
    form_fields = grok.AutoFields(ICustomStudentStudyLevel).omit(
        'level_session', 'level_verdict',
        'validated_by', 'validation_date', 'gpa')

    omit_fields = ('password', 'suspended', 'suspended_comment',
        'phone', 'adm_code', 'sex', 'email', 'date_of_birth',
        'department', 'current_mode')

    @property
    def title(self):
        portal_language = getUtility(IKofaUtils).PORTAL_LANGUAGE
        return translate(_('Credits Registered'), 'waeup.kofa',
            target_language=portal_language)

    def _signatures(self):
        return (
            [('I have selected the course on the advise of my Head of '
             'Department. <br>', _('Student\'s Signature'), '<br>')],
            [('This student has satisfied the department\'s requirements. '
             'I recommend to approve the course registration. <br>',
             _('Head of Department\'s Signature'), '<br>')],
            [('' , _('Principal Assistant Registrar\'s Signature'), '<br>')],
            [('', _('Director\'s Signature'))]
            )

    def render(self):
        portal_language = getUtility(IKofaUtils).PORTAL_LANGUAGE
        Sem = translate('Sem.', 'waeup.kofa', target_language=portal_language)
        Code = translate('Code', 'waeup.kofa', target_language=portal_language)
        Title = translate('Title', 'waeup.kofa', target_language=portal_language)
        Cred = translate('Cred.', 'waeup.kofa', target_language=portal_language)
        Score = translate('Score', 'waeup.kofa', target_language=portal_language)
        Grade = translate('Grade', 'waeup.kofa', target_language=portal_language)
        Signature = translate(_('HOD\'s Signature'), 'waeup.kwarapoly',
            target_language=portal_language)
        studentview = StudentBasePDFFormPage(self.context.student,
            self.request, self.omit_fields)
        students_utils = getUtility(IStudentsUtils)

        tabledata = []
        tableheader = []
        contenttitle = []
        for i in range(1,7):
            tabledata.append(sorted(
                [value for value in self.context.values() if value.semester == i],
                key=lambda value: str(value.semester) + value.code))
            tableheader.append([(Code,'code', 2.0),
                               (Title,'title', 7),
                               (Cred, 'credits', 1.5),
                               (Score, 'score', 1.4),
                               (Grade, 'grade', 1.4),
                               (Signature, 'dummy', 3),
                               ])
        if len(self.label.split('\n')) == 3:
            topMargin = 1.9
        elif len(self.label.split('\n')) == 2:
            topMargin = 1.7
        else:
            topMargin = 1.5
        return students_utils.renderPDF(
            self, 'course_registration_slip.pdf',
            self.context.student, studentview,
            tableheader=tableheader,
            tabledata=tabledata,
            signatures=self._signatures(),
            topMargin=topMargin,
            omit_fields=self.omit_fields
            )